@@ -42,11 +42,20 @@
 
 #include "../precomp.hpp"
 #include "layers_common.hpp"
+#include "../op_cuda.hpp"
 #include "../op_inf_engine.hpp"
 #include "../ie_ngraph.hpp"
+#include "../op_webnn.hpp"
+#include "../op_timvx.hpp"
+#include "../op_cann.hpp"
 
 #include <opencv2/dnn/shape_utils.hpp>
 
+#ifdef HAVE_CUDA
+#include "../cuda4dnn/primitives/reshape.hpp"
+using namespace cv::dnn::cuda4dnn;
+#endif
+
 namespace cv
 {
 namespace dnn
@@ -155,11 +164,14 @@ public:
     ReshapeLayerImpl(const LayerParams& params)
     {
         setParamsFrom(params);
-        int axis = params.get<int>("axis", 0);
-        int numAxes = params.get<int>("num_axes", -1);
+        axis = params.get<int>("axis", 0);
+        numAxes = params.get<int>("num_axes", -1);
         hasDynamicShapes = params.get<bool>("has_dynamic_shapes", false);
         shapesInitialized = !hasDynamicShapes;
 
+        zeropoint = params.get<int>("zeropoints", 0);
+        scale = params.get<float>("scales", 1.0f);
+
         CV_Assert(numAxes >= -1);
         newShapeRange = (numAxes == -1) ? Range(axis, INT_MAX) : Range(axis, axis + numAxes);
 
@@ -195,11 +207,26 @@ public:
 
     virtual bool supportBackend(int backendId) CV_OVERRIDE
     {
+        if (backendId == DNN_BACKEND_TIMVX && haveTimVX())
+        {
+            int len = this->type.length();
+            if (len <= 4)
+                return false;
+
+            if (this->type.substr(len - 4) == "Int8")
+                return true;
+            else
+                return false;
+        }
+
 #ifdef HAVE_INF_ENGINE
         if (backendId == DNN_BACKEND_INFERENCE_ENGINE_NGRAPH)
             return true;
 #endif
-        return backendId == DNN_BACKEND_OPENCV;
+        return backendId == DNN_BACKEND_OPENCV ||
+               backendId == DNN_BACKEND_CUDA ||
+               backendId == DNN_BACKEND_WEBNN ||
+               backendId == DNN_BACKEND_CANN;
     }
 
     bool getMemoryShapes(const std::vector<MatShape> &inputs,
@@ -299,6 +326,39 @@ public:
         }
     }
 
+#ifdef HAVE_CANN
+    virtual Ptr<BackendNode> initCann(const std::vector<Ptr<BackendWrapper> > &inputsWrapper, const int index, const std::vector<Ptr<BackendNode> >& nodes) CV_OVERRIDE
+    {
+        auto x = inputsWrapper[0].dynamicCast<CannBackendWrapper>();
+
+        // create operator
+        std::string op_name = cv::format("reshape_%d", index);
+        auto op = std::make_shared<ge::op::Reshape>(op_name);
+
+        // set attributes
+        op->set_attr_axis(axis);
+        op->set_attr_num_axes(numAxes);
+
+        // set inputs
+        // set inputs : x
+        auto op_x = nodes[0].dynamicCast<CannBackendNode>()->getOp();
+        op->set_input_x_by_name(*op_x, "y");
+        auto x_desc = x->getTensorDesc();
+        op->update_input_desc_x(*x_desc);
+        // set inputs : shape
+        std::vector<int> shape_of_shape{(int)newShapeDesc.size()};
+        Mat shape_mat(shape_of_shape, CV_32S, newShapeDesc.data());
+        auto op_const_shape = std::make_shared<CannConstOp>(shape_mat.data, shape_mat.type(), shape_of_shape, cv::format("%s_shape", op_name.c_str()));
+        op->set_input_shape(*(op_const_shape->getOp()));
+        op->update_input_desc_shape(*(op_const_shape->getTensorDesc()));
+
+        // set outputs
+        auto output_y_desc = std::make_shared<ge::TensorDesc>(ge::Shape(), ge::FORMAT_NCHW, ge::DT_FLOAT);
+        op->update_output_desc_y(*output_y_desc);
+
+        return Ptr<BackendNode>(new CannBackendNode(op));
+    }
+#endif // HAVE_CANN
 
 #ifdef HAVE_DNN_NGRAPH
     virtual Ptr<BackendNode> initNgraph(const std::vector<Ptr<BackendWrapper> >& inputs,
@@ -315,12 +375,139 @@ public:
     }
 #endif  // HAVE_DNN_NGRAPH
 
+#ifdef HAVE_WEBNN
+    virtual Ptr<BackendNode> initWebnn(const std::vector<Ptr<BackendWrapper> >& inputs, const std::vector<Ptr<BackendNode> >& nodes) CV_OVERRIDE
+    {
+        Ptr<WebnnBackendNode> node = nodes[0].dynamicCast<WebnnBackendNode>();
+        auto& webnnInpOperand = node->operand;
+        auto& webnnGraphBuilder = node->net->builder;
+        const std::vector<int32_t> out(outShapes[0].begin(), outShapes[0].end());
+        auto operand = webnnGraphBuilder.Reshape(webnnInpOperand, out.data(), out.size());
+        return Ptr<BackendNode>(new WebnnBackendNode(operand));
+    }
+#endif
+
+#ifdef HAVE_CUDA
+    Ptr<BackendNode> initCUDA(
+        void *context_,
+        const std::vector<Ptr<BackendWrapper>>& inputs,
+        const std::vector<Ptr<BackendWrapper>>& outputs
+    ) override
+    {
+        auto context = reinterpret_cast<csl::CSLContext*>(context_);
+        return make_cuda_node<cuda4dnn::ReshapeOp>(preferableTarget, std::move(context->stream));
+    }
+#endif
+
+    virtual Ptr<BackendNode> initTimVX(void* timVXInfo_,
+                                       const std::vector<Ptr<BackendWrapper> > &inputsWrapper,
+                                       const std::vector<Ptr<BackendWrapper> > &outputsWrapper,
+                                       bool isLast) CV_OVERRIDE
+    {
+#ifdef HAVE_TIMVX
+        // tvGraph Initialization.
+        auto timVxInfo = reinterpret_cast<TimVXInfo *>(timVXInfo_);
+        CV_Assert(timVxInfo);
+        Ptr<TimVXGraph> tvGraph = timVxInfo->getGraph();
+        CV_Assert(tvGraph);
+        Ptr<tim::vx::Graph> graph = tvGraph->graph;
+
+        std::vector<int> inputsIndex, outputsIndex;
+        int input_index = -1, output_index = -1;
+
+        int reshapeNum = 0;
+        Ptr<TimVXBackendWrapper> tmpWrapper, inputWrapper, outputWrapper;
+        for (size_t i = 0; i < outputsWrapper.size(); i++)
+        {
+            tmpWrapper = inputsWrapper[i].dynamicCast<TimVXBackendWrapper>();
+            Mat srcBlob = tmpWrapper->getMat();
+
+            tmpWrapper = outputsWrapper[i].dynamicCast<TimVXBackendWrapper>();
+            Mat dstBlob = tmpWrapper->getMat();
+            if (dstBlob.data != srcBlob.data)
+            {
+                reshapeNum++;
+                inputWrapper = inputsWrapper[i].dynamicCast<TimVXBackendWrapper>();
+                outputWrapper = outputsWrapper[i].dynamicCast<TimVXBackendWrapper>();
+            }
+        }
+
+        // Only work for single reshape Mat
+        if (reshapeNum != 1)
+        {
+          return Ptr<BackendNode>();
+        }
+
+        // Input
+        if (inputWrapper->isTensor())
+        {
+          input_index = tvGraph->getTensorIndex(inputWrapper->getTensor());
+          if (input_index == -1)
+          {
+              // Copy To New inputWrapper
+              Mat tmp = inputWrapper->getMat();
+              inputWrapper = Ptr<TimVXBackendWrapper>(new TimVXBackendWrapper(tmp));
+          }
+        }
+
+        if (!inputWrapper->isTensor() || input_index == -1)
+        {
+            Ptr<tim::vx::Quantization> tvInputQuant = Ptr<tim::vx::Quantization>(
+                    new tim::vx::Quantization(tim::vx::QuantType::ASYMMETRIC, scale, zeropoint));
+          inputWrapper->createTensor(graph,tim::vx::TensorAttribute::INPUT,tvInputQuant);
+          input_index = tvGraph->addWrapper(inputWrapper);
+        }
+        inputsIndex.push_back(input_index);
+
+        //Output
+        // Output Tensor has the same quantized attrib as Input Tesor.
+        Ptr<tim::vx::Quantization> outputQuant = inputWrapper->getTensorQuantization();
+        if (isLast)
+        {
+            auto shapeType = getShapeTypeFromMat(outputWrapper->getMat());
+
+            // For Graph Output tensor, we need to set tensor shape before createTensor().
+            outputWrapper->setTensorShape(shapeType);
+            outputWrapper->createTensor(graph, tim::vx::TensorAttribute::OUTPUT, outputQuant);
+        }
+        else
+        {
+            outputWrapper->createTensor(graph, tim::vx::TensorAttribute::TRANSIENT, outputQuant);
+        }
+        output_index = tvGraph->addWrapper(outputWrapper);
+        outputsIndex.push_back(output_index);
+
+        // generate output shape.
+        MatShape outputShape = shape(outputWrapper->getMat());
+        // reverse shape, from NCHW to WHCN
+        std::reverse(outputShape.begin(), outputShape.end());
+        std::vector<uint32_t> tvShape(outputShape.begin(), outputShape.end());
+
+        std::shared_ptr<tim::vx::Operation> tvReshape = graph->CreateOperation<tim::vx::ops::Reshape>(tvShape);
+
+        Ptr<TimVXBackendNode> tvBackendNode = new TimVXBackendNode(tvGraph, tvReshape, inputsIndex, outputsIndex);
+
+        return tvBackendNode;
+#endif  // HAVE_TIMVX
+        return Ptr<BackendNode>();
+    }
+
+    virtual bool tryQuantize(const std::vector<std::vector<float> > &scales,
+                             const std::vector<std::vector<int> > &zeropoints, LayerParams& params) CV_OVERRIDE
+    {
+        return true;
+    }
+
 private:
+    int axis;
+    int numAxes;
     std::vector<MatShape> outShapes;
     std::vector<int> dynamicShapes; // Which axes shapes are dynamic and require reinitialization with new input
     std::vector<int> inputIndices; // Which axes from input are needed to compute correct output shape
     bool hasDynamicShapes;
     bool shapesInitialized;
+    float scale;
+    int zeropoint;
 };
 
 Ptr<ReshapeLayer> ReshapeLayer::create(const LayerParams& params)
